<?php
/*
Plugin Name: Collaboration
Plugin URI: http://playforward.net/togetherjs
Description: A plugin that integrates TogtherJS in to the dashboard of WordPress.
Version: 1.0
Author: Playforward | Dustin Dempsey
Author URI: http://playforward.net
License: GPLv2 or later
*/

	if ( !class_exists( 'together_js' ) ) {
	
		class together_js {
		
			public $options 	= array();
			public $settings 	= array();
			
			public $terms = array();
			
			function __construct() {
			
				$this->terms = $this->populate_terms();
				
				if ( is_admin() ) {
					add_action( 'admin_menu', array( &$this, 'add_admin_menu' ) );
				}
				
				$this->options 		= $this->populate_options();
				$this->settings 	= $this->populate_settings();
				
				$this->replace_terms();
				
				if ( current_user_can_for_blog( $this->settings['site_role'] ) ) {
				
					add_action( 'admin_bar_menu', array( &$this, 'add_colaborate_menu' ), 0 );
					add_action( 'admin_head', array( &$this, 'add_head' ), 0 );
					add_action( 'wp_head', array( &$this, 'add_head' ), 0 );
					add_action( 'admin_init', array( &$this, 'add_javascripts' ), 0 );
				}
			}
			
			function populate_terms() {
			
				$terms = array();
				
				$terms['option_id']			= 'togetherjs_options';
				$terms['name_menu'] 		= __( 'Collaborate', 'togetherjs' );
				$terms['name_menu_open'] 	= __( 'Hide Collaborate', 'togetherjs' );
				
				$terms = apply_filters( 'togetherjs_terms', $terms );
				
				return $terms;
			}
			
			function replace_terms() {
			
				foreach ( $this->options as $option ) {
					if ( !empty( $option['term'] ) && !empty( $this->settings[$option['id']] ) ) {
						$this->terms[$option['term']] = $this->settings[$option['id']];
					}
				}
			}
			
			function populate_settings() {
			
				$option = get_option( $this->terms['option_id'] );
				
				$settings = array();
				
				if ( !empty( $option ) ) {
					if ( is_serialized( $option ) ) {
						$settings = unserialize( $option );
					} elseif ( is_array( $option ) ) {
						$settings = $option;
					}
				}
				
				if ( empty( $this->options ) ) {
					$this->options = $this->populate_options();
				}
				
				foreach ( $this->options as $option ) {
				
					if ( empty( $settings[$option['id']] ) ) {
					
						$default = '';
						
						if ( !empty( $option['default'] ) ) {
							$default = $option['default'];
						}
						
						$settings[$option['id']] = $default;
					}
				}
				
				$settings = apply_filters( 'togetherjs_settings', $settings );
				
				return $settings;
			}
			
			function populate_options() {
			
				$options = array();
				
				
				$options[] = array(
					'type'	=> 'heading',
					'title'	=> __( 'Main Options', 'togetherjs' )
					);
				
				$options[] = array(
					'id'	=> 'menu_text',
					'type'	=> 'text',
					'title'	=> __( 'Menu Text', 'togetherjs' ),
					'desc'	=> __( 'Change the name of the TogetherJS collaboration tool in the menu. Default is <strong>' . $this->terms['name_menu'] . '</strong>.', 'togetherjs' ),
					'term'	=> 'name_menu',
					'config'	=> array(
						'key'		=> 'TogetherJSConfig_toolName',
						'default'	=> $this->terms['name_menu']
						)
					);
				
				$options[] = array(
					'id'	=> 'menu_text_open',
					'type'	=> 'text',
					'title'	=> __( 'Open Menu Text', 'togetherjs' ),
					'desc'	=> __( 'Change the name of the TogetherJS collaboration tool in the menu when it\'s opened. Default is <strong>' . $this->terms['name_menu_open'] . '</strong>.', 'togetherjs' ),
					'term'	=> 'name_menu_open'
					);
				
				$options[] = array(
					'id'		=> 'site_name',
					'type'		=> 'text',
					'title'		=> __( 'Change Site Name', 'togetherjs' ),
					'desc'		=> __( 'Change the name of the site referenced in TogetherJS. Default is <strong>' . get_bloginfo( 'name' ) . '</strong>.', 'togetherjs' ),
					'config'	=> array( 
						'key' 		=> 'TogetherJSConfig_siteName',
						'default' 	=> get_bloginfo( 'name' )
						)
					);
				
				global $wp_roles;
				$roles = $wp_roles->get_names();
				
				$options[] = array(
					'id'		=> 'site_role',
					'type'		=> 'select',
					'title'		=> __( 'Minimum Role', 'togetherjs' ),
					'desc'		=> __( 'What is the minimum role required to use TogetherJS? Default is <strong>subscriber</strong>.', 'togetherjs' ),
					'default'	=> 'subscriber',
					'options'	=> $roles
					);
				
				$options[] = array(
					'type'	=> 'heading',
					'title'	=> __( 'Advanced Options', 'togetherjs' )
					);
				
				$options[] = array(
					'id'		=> 'hub_base',
					'type'		=> 'text',
					'title'		=> __( 'TogetherJS Hub Base', 'togetherjs' ),
					'desc'		=> __( 'Want to use a custom hub for message relays? Define it here. The default uses Mozilla\'s servers.', 'togetherjs' ),
					'config'	=> array( 
						'key' => 'TogetherJSConfig_hubBase'
						)
					);
				
				$options[] = array(
					'id'		=> 'click_selector',
					'type'		=> 'text',
					'title'		=> __( 'Clone Click Selector', 'togetherjs' ),
					'desc'		=> __( 'This should be a jQuery CSS selector. Clicks on elements that match this selector will be repeated across all users.', 'togetherjs' ),
					'config'	=> array( 
						'key' => 'TogetherJSConfig_cloneClicks'
						)
					);
				
				$options[] = array(
					'id'		=> 'keyboard_shortcut',
					'type'		=> 'select',
					'title'		=> __( 'Enable Keyboard Shortcut', 'togetherjs' ),
					'desc'		=> __( 'If enabled, you can launch TogetherJS using alt+T alt+T (twice).', 'togetherjs' ),
					'config'	=> array( 
						'key' 		=> 'TogetherJSConfig_enableShortcut',
						'default'	=> 'false'
						),
					'options'	=> array(
						'false' => 'Disabled',
						'true' => 'Enabled'
						)
					);
				
				$options[] = array(
					'id'		=> 'invite_flag',
					'type'		=> 'select',
					'title'		=> __( 'Invite Prompt', 'togetherjs' ),
					'desc'		=> __( 'When a user starts a session they can be prompted to invite others. Set to enabled to show this prompt.', 'togetherjs' ),
					'config'	=> array( 
						'key' 		=> 'TogetherJSConfig_suppressInvite',
						'default'	=> 'true'
						),
					'options'	=> array(
						'true' 	=> 'Disabled',
						'false' => 'Enabled'
						)
					);
				
				$options[] = array(
					'id'		=> 'hash_flag',
					'type'		=> 'select',
					'title'		=> __( 'Respect Hashes', 'togetherjs' ),
					'desc'		=> __( 'Should URLs with different hash tags be considered different pages?', 'togetherjs' ),
					'config'	=> array( 
						'key' 		=> 'TogetherJSConfig_includeHashInUrl',
						'default'	=> 'false'
						),
					'options'	=> array(
						'false' => 'No',
						'true' 	=> 'Yes'
						)
					);
				
				$options = apply_filters( 'togetherjs_options', $options );
				
				return $options;
			}
			
			function render_field( $option = null ) {
			
				if ( empty( $option ) || empty( $option['type'] ) ) {
					return;
				}
				
				$title 	= $option['title'];
				$id 	= $option['id'];
				$desc	= $option['desc'];
				
				$value 	= $this->settings[$id];
				
				if ( !empty( $desc ) ) {
					$desc = '<p class="description">' . $desc . '</p>';
				}
				
				if ( $option['type'] == 'heading' ) {
					echo '
						<tr valign="top">
							<th colspan="2" scope="row"><h3>' . $title . '</h3></th>
						</tr>
						';
				} elseif ( $option['type'] == 'mainheading' ) {
					echo '
						<tr valign="top">
							<th colspan="2" scope="row"><h2>' . $title . '</h2></th>
						</tr>
						';
				} elseif ( $option['type'] == 'text' ) {
					echo '
						<tr valign="top">
							<th scope="row"><label for="' . $id . '">' . $title . '</label></th>
							<td><input name="' . $id . '" type="text" id="' . $id . '" value="' . $value . '" class="regular-text">' . $desc . '</td>
						</tr>
						';
				} elseif ( $option['type'] == 'select' ) {
				
					$opts = '';
					
					foreach ( $option['options'] as $key => $val ) {
					
						$selected = '';
						
						if ( $value == $key ) {
							$selected = ' selected="selected" ';
						}
						
						$opts .= '<option value="' . $key .'" ' . $selected . '>' . $val . '</option>';
					}
					
					echo '
						<tr valign="top">
							<th scope="row"><label for="' . $id . '">' . $title . '</label></th>
							<td>
								<select name="' . $id . '" id="' . $id . '">
									' . $opts . '
								</select>
								' . $desc . '</td>
							
						</tr>
						';
				}
			}
			
			function add_colaborate_menu() {
			
				global $wp_admin_bar;
				
				$options = array(
					'id'   		=> 'togetherjs',
					'parent' 	=> 'top-secondary',
					'title' 	=> __( $this->terms['name_menu'], 'togetherjs' ),
					'href' 		=> '#',
					'meta' 		=> array( 
						'html' 		=> false, 
						'class' 	=> 'togetherjs-button', 
						'onclick' 	=> '', 
						'target' 	=> '', 
						'title' 	=> __( $this->terms['name_menu'], 'togetherjs' ),
						'tabindex' 	=> -1
						)
					);
				
				$wp_admin_bar->add_menu( $options );
			}
			
			function add_head() {
			
				global $current_user;
				
				$site_id			= get_current_blog_id();
				
				$user_name 			= $current_user->display_name;
				$user_avatar 		= $this->get_gravatar_url( $current_user->user_email );
				
				$name_menu			= $this->terms['name_menu'];
				$name_menu_open		= $this->terms['name_menu_open'];
				
				$configs = '';
				
				foreach ( $this->options as $option ) {
				
					if ( !empty( $option['config'] ) ) {
					
						if ( !empty( $option['config']['key'] ) ) {
						
							$default_value = '';
							
							if ( !empty( $option['config']['default'] ) ) {
							
								$default_value = $option['config']['default'];
							}
							
							if ( !empty( $this->settings[$option['id']] ) ) {
								$configs .= ' var ' . $option['config']['key'] . ' = "' . $this->settings[$option['id']] . '";';
							} elseif ( !empty( $default_value ) ) {
								$configs .= ' var ' . $option['config']['key'] . ' = "' . $default_value . '";';
							}
						}
					}
				}
				
				echo '
					<script type="text/javascript">
					
						var TogetherJSConfig_suppressInvite		= true;
						var TogetherJSConfig_getUserName 		= function () {return "' . $user_name . '";};
						var TogetherJSConfig_getUserAvatar  	= function () {return "' . $user_avatar . '";};
						var TogetherJSConfig_findRoom			= "site_' . $site_id . '";
						
						' . $configs . '
						
						jQuery( document ).ready( function( $ ) {
							
							TogetherJS.on( "close", function () {
								$("#wp-admin-bar-togetherjs").children().first().html("' . __( $name_menu, 'togetherjs' ) . '");
							});
							
							TogetherJS.on( "ready", function () {
								$("#wp-admin-bar-togetherjs").children().first().html("' . __( $name_menu_open, 'togetherjs' ) . '");
							});
							
							$( "#wp-admin-bar-togetherjs" ).click( function( event ) {
							
								event.preventDefault();
								TogetherJS(this);
								return false;
							});
						});
					</script>
					';
			}
			
			function add_javascripts() {
			
				wp_register_script( 'togetherjs', 'https://togetherjs.com/togetherjs-min.js', array( 'jquery' ), '1.0', true );
				wp_enqueue_script( 'togetherjs' );
			}
			
			function get_gravatar_url( $email, $size = 40 ) {
			
				$hash = md5( strtolower( trim ( $email ) ) );
				return 'http://gravatar.com/avatar/' . $hash . '&s=' . $size;
			}
			
			function add_admin_menu() {
				add_options_page( __( 'Collaborate Options', 'togetherjs' ), __( $this->terms['name_menu'], 'togetherjs' ), 'manage_options', 'togetherjs', array( &$this, 'options_page' ) );
			}
			
			function options_page() {
			
				if ( !current_user_can( 'manage_options' ) )  {
					wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
				}
				
				if ( !empty( $_POST['saved'] ) && wp_verify_nonce( $_POST['togetherjs_field'], 'togetherjs_action' ) ) {
				
					foreach ( $this->options as $key => $option ) {
						if ( isset( $_POST[$option['id']] ) ) {
							$this->settings[$option['id']] = $_POST[$option['id']];
						}
					}
					
					$settings_array = array();
					
					foreach ( $this->settings as $key => $value ) {
						if ( !empty( $key ) && !empty( $value ) ) {
							$settings_array[$key] = $value;
						}
					}
					
					if ( get_option( $this->terms['option_id'] ) !== false ) {
						$return = update_option( $this->terms['option_id'], $settings_array );
					} else {
						$return = add_option( $this->terms['option_id'], $settings_array );
					}
				}
				
				echo '<div class="wrap">';
				
					echo '<h2>' . __( $this->terms['name_menu'] . ' Options', 'togetherjs' ) . '</h2>';
					
					echo '<form method="post">';
					
						echo '<input type="hidden" name="saved" value="1" />';
						
						wp_nonce_field( 'togetherjs_action', 'togetherjs_field' );
						
						echo '<table class="form-table"><tbody>';
						
							foreach ( $this->options as $option ) {
								$this->render_field( $option );
							}
						
						echo '</tbody></table>';
						
						echo '<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes"></p>';
					
					echo '</form>';
				
				echo '
					<p>For more information, visit <a href="https://togetherjs.com/" title="TogetherJS">TogetherJS.com</a>.</p>
					<p>For support, please use the plugin\'s support forum at <a href="http://wordpress.org/plugins/collaboration/" title="WordPress">WordPress.org</a>.</p>
					<p class="together_js_credit">This plugin is built by Dustin Dempsey of <a href="http://playforward.net/?ref=togetherjs" title="Playforward Web Development">Playforward</a> and it is not affiliated with <a href="http://mozillalabs.com" title="Mozilla labs">Mozilla labs</a>.</p>
					';
				
				echo '</div>';
			}
		}
		
		function together_js_init() {
		
			if ( is_admin_bar_showing() && is_user_logged_in() ) {
			
				global $together_js;
				$together_js = new together_js();
			}
		}
		
		add_action( "init", "together_js_init" );
	}

?>